****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_finalize_encmasterlist.sas  
*
* Created (mm/dd/yyyy): 02/01/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program will finalize the numerator dataset by doing final calculation and variables assessment 
*
*  Program inputs:                                                                                   
*	-Dataset with valid numerator observations (_EncMasterList)
* 
*  Program outputs:                                                                                                                                       
* 	-Dataset with valid numerator observations (_EncMasterList)
* 
* PARAMETERS:
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_finalize_encmasterlist();
	
%put =====> MACRO CALLED: ms_finalize_encmasterlist v1.0;

	data _EncMasterList;
  		merge _EncMasterList (in=a)
	          _encounterswithevents_byID;
	    by patid encounterID;
		if a;

		format Group $30. bloodcomp $65.;
	    Group="&ITGROUP."; 
	    Year=Year(enc_start);

		*fill in missing event values with 0;
		%if %eval(&num_hoi.>0) %then %do;
			array hoi{&num_hoi.} event1-event&num_hoi.;
			do i = 1 to dim(hoi);
				if missing(hoi{i}) then hoi{i} = 0;
			end;
			drop i;
		%end;

		*calculate LOS;
			if upcase(enctype) in ('IP', 'IS') then do;
				los = coalesce(enc_end,enc_start) - enc_start + 1; 
			end;
			else do;
				los = .M;
			end;
			
		*Combine IP and IS into IP;
		if enctype = 'IS' then enctype = 'IP';

		*Reclassify missing discharge_disposition as U;
		if missing(discharge_disposition) then discharge_disposition='U';

		*initialize transfusion/blood component vars;
			plate_any = 0;
			rbc_any = 0;
			plasma_any = 0;
			other_bld = 0;
			bloodcomp = '';
			trans_unit = 0;

		rename enc_start = adate
			   enc_end = ddate;
	run;
	
	*Calculate Transfusion/Blood components;

		*Restrict to only codes during valid encounters;
		proc sql noprint;
			create table _grouputil as
			select grp.*
			from _groupindex as grp
			where encounterID in 
				(select distinct encounterID from _EncMasterList)
			and codecat = 'TR';
		quit;

		%isdata(dataset=_grouputil);
		%if %eval(&nobs.>0) %then %do;

			%isdata(dataset=transcodes);
			%if %eval(&nobs.>0) %then %do;

				*count number of transfusions - 
				all transfusions regardless of whether they are defined as an exposure, so need to reference SCDM Transfusion table;
				proc sql noprint;
					create table _alltrans as
					select trans.patid,
						   trans.encounterID,
						   1 as trans
					from indata.&ITRANSTABLE. as trans,
					_EncMasterList as enc
					where trans.patid = enc.patid and trans.encounterID = enc.encounterID;
				quit;

				proc means noprint nway data=_alltrans;
					var trans;
					class patid encounterID;
					output out=_transunit(drop=_:) n=trans_unit;
				run;

				proc sql noprint;
					create table _linkedtranscodes as
					select util.patid,
						   util.encounterID,
						   util.code,
						   util.codetype,
					
						   case when upcase(substr(requirement,1,4)) ='PLAT' then 1
						   else 0
						   end as PLATE,

						   case when upcase(substr(requirement,1,3)) ='RBC' then 1
						   else 0
						   end as RBCS,

						   case when upcase(substr(requirement,1,4)) ='PLAS' then 1
						   else 0
						   end as PLASMA,

						   case when upcase(requirement) ='CRYO' |  upcase(requirement) ='OTHER' | missing(requirement) then 1
						   else 0
						   end as OTHER

					from _grouputil as util
					left join transcodes as trans
					on util.code=trans.code and util.codetype = trans.codetype
					order by patid, encounterID;
				quit;

				*Summarize to 1 per encounter;
				proc means noprint nway data=_linkedtranscodes;
					var plate rbcs plasma other;
					class patid encounterID;
					output out=_bloodperenc(drop=_:) max=;
				run;

				*Merge with _EncMasterList;
				data _EncMasterList;
					merge _EncMasterList (in=a)
						  _bloodperenc (in=b)
						  _transunit;
					by patid encounterID;
					if a;

					if plate = 1 then plate_any = 1; else plate_any = 0;
					if rbcs = 1 then rbc_any = 1; else rbc_any = 0;
					if plasma = 1 then plasma_any = 1; else plasma_any = 0;
					if other = 1 then other_bld = 1; else other_bld = 0;

					if trans_unit = . then trans_unit = 0;

					*compute BloodComp;
					if rbc_any=1 and (plate_any^=1 & plasma_any^=1 & other_bld^=1) then bloodcomp = 'RBCs only';
					else if plasma_any=1 and (plate_any^=1 & rbc_any^=1 & other_bld^=1) then bloodcomp = 'Plasma only';
					else if plate_any=1 and (rbc_any^=1 & plasma_any^=1 & other_bld^=1) then bloodcomp = 'Platelets only';
					else if plate_any=1 & plasma_any=1 and (rbc_any^=1 & other_bld^=1) then bloodcomp = 'Platelets and Plasma';
					else if rbc_any=1 & plasma_any=1 and (plate_any^=1 & other_bld^=1) then bloodcomp = 'RBCs and Plasma';
					else if rbc_any=1 & plate_any=1 and (plasma_any^=1 & other_bld^=1) then bloodcomp =  'RBCs and Platelets';
					else if rbc_any=1 & plate_any=1 & plasma_any=1 & other_bld^=1  then bloodcomp ='RBCs and Plasma and Platelets';
					else if other_bld=1 then bloodcomp = 'Other';

					drop plate rbcs plasma other;
				run;
			%end;
			%else %do;
				%put WARNING: One or more TRANSFUSION lookup tables is not specified correctly;
			%end;
		%end;
  
	*Remove all labels;
    proc datasets lib=work memtype=data nolist nowarn;
    modify _EncMasterList; 
    attrib _all_ label=' '; 
    run;

%put NOTE: ********END OF MACRO: ms_finalize_encmasterlist v1.0********;

%mend ms_finalize_encmasterlist;
