****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_qrpecov.sas  
*
* Created (mm/dd/yyyy): 03/01/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will extract covariates.                                        
*   
*  Program inputs:  
*	-dplocal.&RUNID._all_encounter
*	-infolder.&COHORTFILE. 
*	-infolder.&Covariatecodes.
*                        
*  Program outputs: 
*	-dplocal.&RUNID._all_covariates
*
*  PARAMETERS:       
*            
*  Programming Notes:                                                                                
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_qrpecov();

	%put =====> MACRO CALLED: ms_qrpecov v1.0;

	*Must have at least 1 covariate;
	%ISDATA(dataset=infolder.&COVARIATECODES.);	    
	%IF %EVAL(&NOBS.>0) %THEN %DO;

		/******************/
		/* Pre-Processing */
		/******************/

		*List of encounterIDs from qrpenum;
		proc sort nodupkey data=DPLocal.&RUNID._all_encounter out=EncList(keep=EncounterID);
			by EncounterID;
		run;

		proc sort nodupkey data=DPLocal.&RUNID._all_encounter out=&RUNID._all_encounter;
		   	by Group Patid encounterID adate;
		run;

		*must have at least one patient;
		%isdata(dataset=&RUNID._all_encounter);
		%if %eval(&NOBS. > 0) %then %do;

			%ISDATA(dataset=infolder.&COVARIATECODES.);	    
			%IF %EVAL(&NOBS.>0) %THEN %DO;					
				proc sql noprint;	
				    select count (distinct CovarNum) 
				    into :NumCovars
				    from infolder.&COVARIATECODES.
					where codecat not in ("CC");
				quit;
				proc sql noprint;	
				    select count (distinct CovarNum) 
				    into :NumComb
				    from infolder.&COVARIATECODES.
					where codecat in ("CC");
				quit;
				%put Number of covars: &NumCovars;
				%put Number of covarComb: &NumComb;
			%END;


			/****************************/
			/* Extract Covariate data   */ 
			/****************************/
			%macro RunCovExtraction;

		        data _CovinpatRX
		             _CovMeds
					 _CovTrans;
			        set CovarCodes(keep=Code CodeCat Codetype CARESETTINGPRINCIPAL studyname CovarNum covfrom covto covartype padmit);       
			        if CodeCat="IR" then output _CovinpatRX; 
			        if CodeCat in ("DX","PX") then output _CovMeds; 
			        if CodeCat="TR" then output _CovTrans; 
		        run;

				%MS_ProcessWildcards(InFile=_CovMeds, 
			                         CodeVar=code, 
			                         OutFile=_CovMeds);

			    *Break out CaresettingPrincipal into Caresetting and Principal;
			    %ms_caresettingprincipal(InFile=_CovMeds, 
			                             Var=CareSettingPrincipal, 
			                             OutFile=_CovMeds);

			    %ms_caresettingprincipal(InFile=_CovTrans, 
			                             Var=CareSettingPrincipal, 
			                             OutFile=_CovTrans);

				*Break of PAdmit;
			    %ms_padmit(InFile=_CovMeds, 
			               Var=padmit, 
			               OutFile=_CovMeds);

				%ms_padmit(InFile=_CovTrans, 
			               Var=padmit, 
			               OutFile=_CovTrans);

				*Extracting meds using EncType and PDX to meds;
			     %ms_qrpe_loopmeds(datafile=meds,
		                 lookfile=_CovMeds, 
		                 outfile=_temp_PreMeds);

				*restricting to encounters in qrpenum;
			 	data _temp_PreMeds1;
			        if 0 then set EncList;
			        declare hash pt (hashexp:16, dataset:"EncList");
			        pt.definekey('EncounterID');
			        pt.definedone();

			        do until(eof1);
			          set _temp_PreMeds end=eof1;
			          if pt.find()=0 then output;
			        end;
			        stop;
				run;

				*Extracting inpatient RX;
				%ms_extractinpatientdrugs(datafile=inpatndcextract,
	                     datavar=NDC ,
	                     lookfile=_CovinpatRX, 
	                     lookvar=code , 
	                     outfile=_temp_PreInpatDrugs);

				*restricting to pts in cidanum;
			 	data _temp_PreInpatDrugs1;
			        if 0 then set EncList;
			        declare hash pt1 (hashexp:16, dataset:"EncList");
			        pt1.definekey('EncounterID');
			        pt1.definedone();

			        do until(eof1);
			          set _temp_PreInpatDrugs end=eof1;
			          if pt1.find()=0 then output;
			        end;
			        stop;
				run;

				*Extracting inpatient transfusions;
	  			%ms_qrpe_loopmeds(datafile=transextract,
		                 lookfile=_CovTrans, 
		                 outfile=_temp_PreTrans);

				*restricting to encounters in qrpenum;
			 	data _temp_PreTrans1;
			        if 0 then set EncList;
			        declare hash pt (hashexp:16, dataset:"EncList");
			        pt.definekey('EncounterID');
			        pt.definedone();

			        do until(eof1);
			          set _temp_PreTrans end=eof1;
			          if pt.find()=0 then output;
			        end;
			        stop;
				run;

				proc datasets nowarn noprint lib=work; 
					delete _temp_PreMeds _temp_PreInpatDrugs _temp_PreTrans;
				quit;  
			%mend;
			%RunCovExtraction;

        	*Combine records;
	        data _CombineRecords;
				set _temp_PreMeds1(keep=encounterID patid adate covartype studyname covarnum covfrom covto)
					_temp_PreInpatDrugs1(keep=encounterID patid rxadate covartype studyname covarnum covfrom covto rxatime)
					_temp_PreTrans1(keep=encounterID patid adate covartype studyname covarnum covfrom covto);
		        where studyname ne '';
				covdate = coalesce(adate, rxadate);
				format covdate date9.;
			run; 

			proc datasets library=work nowarn nolist;
	        	delete _temp_PreMeds1 _temp_PreInpatDrugs1 _temp_PreTrans1;
	        quit;

		    /*Finding those claims during encounters*/
	        proc sql noprint;
	        create table _Covar as
	        select  enc.Patid,
	                enc.Group,
	                enc.encounterID,
	                rec.covdate,
					rec.covartype,
					rec.studyname,
					rec.covarnum,
					rec.rxatime
	        from &RUNID._all_encounter as enc, 
	             _CombineRecords as Rec
	        where enc.Patid = rec.Patid and enc.encounterID = rec.encounterID and
	             %MS_PeriodsOverlap(period1=coalesce(enc.adate+rec.covFrom,enc.adate) COALESCE(enc.adate+rec.CovTo,enc.ddate),                                             
	                                period2=Rec.covdate)
	        order by enc.Group,
	                 enc.Patid,
	                 enc.encounterID;
	        quit;

		/*_Covar contains 1 observation per occurence of each covariate during the encounter
			Split out COMORB and MEDADM covariate types to calculate Days_rx_allow_gap, days_rx_nogap, and codefreq*/

			proc sort data=_covar(where=(upcase(covartype)='MEDADM')) out=_medadm;
				by group patid encounterID covarnum covdate ;
			run;

			*collapse COMORB to 1 row per group, covarnum, patid, encounterID;
			proc sort data=_covar(where=(upcase(covartype)='COMORB')) out=_comorb nodupkey;
				by group patid encounterID covarnum;
			run;

			%isdata(dataset=_medadm);
			%if %eval(&nobs.>0) %then %do;

				*merge in daysgap parameter from &cohortfile.;
				proc sort data=infolder.&cohortfile. out=_cohortfile;
					by group;
				run;
				data _medadm;
					merge _medadm (in=a)
						  _cohortfile(keep=group daysgap);
					by group;
					if a;
				run;

				*compute days_rx_allow_gap;
				data _allowgap;
					set _medadm;
					by group patid encounterID covarnum;
					if first.covarnum then do;
				    	LRunOutDate=covdate;
				      	episode=1;
				      	gap=.;
				   	end;
					else do; 
	      				gap = covdate - LRunOutDate-1;         

						if (gap > daysgap) then do;
							episode=episode+1; 
				         	LRunOutDate = covdate;		
						end;
	      				else do; 
							LRunOutDate = max(covdate,LRunOutDate); 
						end;
	  				end;
	   
	   				format LRunOutDate date9.;
				   	retain episode LRunOutDate;

					count = 1;
				run;

				*Summarize;
				proc means data=_allowgap nway noprint missing;
					var covdate;
					class group PatId encounterID covarnum studyname covartype Episode;
					output out=_allowgap_summ(drop=_:) 	
								min(covdate)=CovStartDt  
					            max(covdate)=CovEndDt/KEEPLEN;
				run; 

				*days_rx_nogap and codefreq;
				*Apply codefreq_nodup for each group;
				%let codefreqY = ;
				%let codefreqN = ;
				proc sql noprint;
					select distinct quote(trim(left(group))) into: codefreqY separated by ' '
					from _cohortfile
					where codefreq_nodup = 'Y';
					select distinct quote(trim(left(group))) into: codefreqN separated by ' '
					from _cohortfile
					where codefreq_nodup = 'N';
				quit;

				%if %length(&codefreqY.) > 0 | %str(&codefreqY.) ne %str() %then %do;
					proc sort data=_allowgap nodupkey out=_allowgapY;
						by group patid encounterID covarnum studyname covartype episode covdate rxatime count;
						where group in(&codefreqY);
					run;
				%end;
				
				data _allowgap_allgroups;
					set
					%if %length(&codefreqY.) > 0 | %str(&codefreqY.) ne %str() %then %do;
						_allowgapY 
					%end;
					%if %length(&codefreqN.) > 0 | %str(&codefreqN.) ne %str() %then %do;
						_allowgap(where=(group in(&codefreqN)))
					%end;
					;
				run;

				proc means data=_allowgap_allgroups nway noprint missing;
					var count ;
					class group PatId encounterID covarnum studyname covartype Episode covdate ;
					output out=_allowgap_nogap(drop=_type_ rename=_freq_=freq) max=;
				run;
				

				proc means data=_allowgap_nogap nway noprint missing;
					var count freq;
					class group PatId encounterID covarnum studyname covartype;
					output out=_medadm_calc_nogap(drop=_:)
						sum(count) = days_rx_nogap 
						sum(freq) = codefreq;
				run; 									

				*calculate length of pseudo-episode for days_rx_allow_gap;
				data _allowgap_summ;
					set _allowgap_summ;
					days_rx_allow_gap = CovEndDt - CovStartDt+1;
				run;

				*Summarize to count days_rx_allow_gap for each covariate;
				proc means data=_allowgap_summ nway noprint missing;
					var days_rx_allow_gap;
					class group patid encounterID covarnum studyname covartype;
					output out=_medadm_calc_days(drop=_:) sum=;
				run;

				*merge;
				data _medadm_calc;
					merge _medadm_calc_days
						  _medadm_calc_nogap;
					by group patid encounterID covarnum;
				run;
						
				*Comnine with COMORB covariates and save to dplocal;
				data dplocal.&runid._all_covariates;
					set _comorb(in=a drop=covdate)
						_medadm_calc;
						if a then do;
							days_rx_allow_gap = .;
							days_rx_nogap = .;
							codefreq = .;
						end;
				run;
		%end;
		%else %do;
			data dplocal.&runid._all_covariates;
				set _comorb(drop=covdate);
				days_rx_allow_gap = .;
				days_rx_nogap = .;
				codefreq = .;
			run;
		%end;
		%end;
		%else %do;
		%put WARNING: No encounters were identified in this request.;
		%end;
	%end;

	%put NOTE: ******** END OF MACRO: ms_qrpecov v1.0 ********;

%mend ms_qrpecov;
